package com.zanichelli.android.cronometro;

/**
 * Created by Gabriele on 23/11/2016.
 */

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;


public class ChronometerData {
  // stati del cronometro
  public static final int CHRONOMETER_STATE_RESET = 0;
  public static final int CHRONOMETER_STATE_STOPPED = 1;
  public static final int CHRONOMETER_STATE_PAUSED = 2;
  public static final int CHRONOMETER_STATE_RUNNING = 3;

  // formattatore usato per visualizzare i tempi
  private static DateFormat dateFormatter = new SimpleDateFormat("HH:mm:ss.SS");
  // numero di cifre usate per le frazioni di secondo
  private static int precision = 2;

  // restituisce il numero di cifre per le frazioni di secondo
  public static int getPrecision() {
    return precision;
  }

  // imposta il numero di cifre per le frazioni di secondo
  public static void setPrecision(int prec) {
    String format = "HH:mm:ss.";

    switch(prec) {
      case 3:
        format+="S";
      case 2:
        format+="S";
      default:
        format+="S";
        break;
    }

    if (prec > 3 || prec < 1)
      precision = 1;
    else
      precision = prec;
    dateFormatter = new SimpleDateFormat(format);
  }

  public static String format(Date d) {
    return dateFormatter.format(d);
  }

  // stato corrente del cronometro
  private int status;
  // elenco dei giri registrati
  private ArrayList<Date> laps;
  // ultimo tempo di attivazione del cronometro
  private Date lastStartTime;
  // se il cronometro è stato messo in pausa,
  // questa variabile contiene il numero di millisecondi
  // trascorsi prima che il cronometro venisse fermato
  private long offset;

  // azzera il cronometro
  public void reset() {
    status = CHRONOMETER_STATE_RESET;
    laps = new ArrayList<>();
    lastStartTime = null;
    offset = 0L;
  }

  // costruttore
  public ChronometerData() {
    reset();
  }

  // restituisce lo stato del cronometro
  public int getStatus() {
    return status;
  }


  // avvia un cronometro fermo o in pausa.
  // Se il cronometro era stato fermato esegue
  // anche contestualmente l'azzeramento
  public void start() {
    if(status == CHRONOMETER_STATE_RUNNING)
      throw new RuntimeException("You cannot start an already running Chronometer!");

    if(status == CHRONOMETER_STATE_STOPPED)
      reset();

    lastStartTime = new Date();
    status = CHRONOMETER_STATE_RUNNING;
  }

  // ferma il cronometro;
  // un cronometro fermato viene azzerato una volta riavviato
  public void stop() {
    if(status <= CHRONOMETER_STATE_STOPPED)
      throw new RuntimeException("You cannot stop an already stopped Chronometer!");
    status = CHRONOMETER_STATE_STOPPED;
    offset = getValue().getTime();
  }

  // mette in pausa il cronometro
  public void pause() {
    if(status != CHRONOMETER_STATE_RUNNING)
      throw new RuntimeException("You cannot pause a non running Chronometer!");
    offset = getValue().getTime();
    status = CHRONOMETER_STATE_PAUSED;
  }

  // restituisce il valore attuale del cronometro
  public Date getValue() {
    // se il cronometro è fermo, il valore attuale
    // è ottenuto a partire dai millisecondi di offset
    if(status != CHRONOMETER_STATE_RUNNING)
      return new Date(offset);

    Date curTime = new Date();
    long value = curTime.getTime() - lastStartTime.getTime();
    value += offset;

    return new Date(value);
  }

  // restituisce il valore formattato del cronometro
  public String getFormattedValue() {
    return dateFormatter.format(getValue());
  }

  // registra un nuovo giro
  public void addLap() {
    if(status != CHRONOMETER_STATE_RUNNING)
      throw new RuntimeException("You cannot add a lap while Chronometer is not running!");
    laps.add(getValue());
  }

  // restituisce vero se è memorizzato almeno un giro
  public boolean hasLaps() {
    return laps.size() > 0;
  }

  // restituisce l'ultimo giro registrato
  public Date getLastLapTime() {
    if(!hasLaps())
      return new Date(0);
    return laps.get(laps.size() - 1);
  }

  // classe che permette di memorizzare il numero di giro, il valore del cronometro
  // relativo a quel giro e la sua durata
  public static class LapData {
    public final int lap_num;
    public final Date lap_time;
    public final Date lap_duration;
    public LapData(int lap_n, Date lap_t, Date lap_d) {
      lap_num = lap_n;
      lap_time = lap_t;
      lap_duration = lap_d;
    }
  }

  private Date dateDiff(Date a, Date b) {
    return new Date(a.getTime() - b.getTime());
  }

  // restituisce l'elenco completo dei giri registrati sotto forma di array
  public LapData[] getLapsData() {
    if(laps.size() < 1)
      return new LapData[0];

    LapData[] data = new LapData[laps.size()];

    data[0] = new LapData(1, laps.get(0), laps.get(0));

    for(int i = 1; i < data.length; ++i) {
      data[i] = new LapData(i+1,
        laps.get(i),
        dateDiff(laps.get(i), laps.get(i - 1)));
    }

    return data;
  }

}
