package com.zanichelli.android.cronometro;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

public class MainActivity extends AppCompatActivity
  implements View.OnClickListener,
  SharedPreferences.OnSharedPreferenceChangeListener{

  // elementi dell'interfaccia utente
  private TextView counter;
  private TextView lastLapAbsolute;
  private Button startPause;
  private Button lapReset;
  private Button viewLaps;
  private Button settings;


  // gestore della routine che aggiorna la visualizzazione
  private Handler myHandler;
  private UpdateStopwatch runnable;
  private static final long UPDATE_INTERVAL = 42L;

  // stato attuale del cronometro
  private ChronometerApplication app;
  private ChronometerData chronometer;

  // attributi per la gestione delle preferenze
  private AlertDialog dialog;

  // codice di aggiornamento della visualizzazione
  private class UpdateStopwatch implements Runnable {
    @Override
    public void run() {
      if (MainActivity.this.chronometer.getStatus()
        == ChronometerData.CHRONOMETER_STATE_RUNNING) {

        MainActivity.this.updateTime();

        // richiede una nuova attivazione della routine dopo
        // un certo numero di millisecondi
        MainActivity.this.myHandler.postDelayed(this, MainActivity.UPDATE_INTERVAL);
      }
    }
  }

  // aggiorna il valore correntemente visualizzato del cronometro
  private void updateTime() {
    String formattedValue = chronometer.getFormattedValue();
    counter.setText(formattedValue);
  }

  // aggiorna la parte dell'interfaccia che mostra
  // l'ultimo giro e il bottone per visualizzare
  // l'elenco completo dei giri;
  // quando nessun giro è ancora stato registrato
  // questi bottoni vengono nascosti
  private void updateLapUI() {
    if (!chronometer.hasLaps()) {
      // niente da mostrare
      lastLapAbsolute.setVisibility(View.INVISIBLE);
      viewLaps.setVisibility(View.INVISIBLE);
      return;
    }

    // aggiorno l'ultimo giro registrato
    String lastLapTime =
      ChronometerData.format(chronometer.getLastLapTime());

    lastLapAbsolute.setText(
      getString(R.string.last_lap_base) + " " + lastLapTime);

    lastLapAbsolute.setVisibility(View.VISIBLE);
    viewLaps.setVisibility(View.VISIBLE);
  }

  // aggiorna l'interfaccia a seconda dello stato del cronometro
  // a cronometro resettato l'unico pulante disponibile è START;
  // se il cronometro è in esecuzione, i pulsanti rispettivamente
  // mettono in pausa il cronometro o registrano un giro;
  // a cronometro fermo, i pulsanti rispettivamente riavviano o
  // fermano/azzerano il cronometro
  private void updateUI() {
    switch (chronometer.getStatus()) {
      case ChronometerData.CHRONOMETER_STATE_RESET:
        startPause.setText(getString(R.string.start));
        lapReset.setVisibility(View.GONE);
        break;
      case ChronometerData.CHRONOMETER_STATE_STOPPED:
        startPause.setText(getString(R.string.start));
        lapReset.setVisibility(View.VISIBLE);
        lapReset.setText(getString(R.string.reset));
        break;
      case ChronometerData.CHRONOMETER_STATE_PAUSED:
        startPause.setText(getString(R.string.start));
        lapReset.setText(getString(R.string.stop));
        break;
      case ChronometerData.CHRONOMETER_STATE_RUNNING:
        startPause.setText(getString(R.string.pause));
        lapReset.setVisibility(View.VISIBLE);
        lapReset.setText(getString(R.string.lap));
        break;
    }
    updateTime();
    updateLapUI();
  }

  // quando il cronometro è fermo o in pausa questo pulsante
  // lo avvia; quando è in esecuzione lo mette in pausa
  private void onStartPauseClick() {
    switch (chronometer.getStatus()) {
      case ChronometerData.CHRONOMETER_STATE_RUNNING:
        myHandler.removeCallbacks(runnable);
        chronometer.pause();
        updateTime();
        break;
      case ChronometerData.CHRONOMETER_STATE_RESET:
      case ChronometerData.CHRONOMETER_STATE_STOPPED:
      case ChronometerData.CHRONOMETER_STATE_PAUSED:
        chronometer.start();
        // la routine verrà attivata dopo un certo numero di millisecondi
        myHandler.postDelayed(runnable, UPDATE_INTERVAL);
        break;
    }

    updateUI();
  }

  // quando il cronometro è in esecuzione questo pulsante
  // registra un nuovo giro; quando è in pausa ferma
  // definitivamente il cronometro; quando è fermo lo azzera
  private void onLapResetClick() {
    switch (chronometer.getStatus()) {
      case ChronometerData.CHRONOMETER_STATE_RUNNING:
        chronometer.addLap();
        break;
      case ChronometerData.CHRONOMETER_STATE_PAUSED:
        chronometer.stop();
        break;
      case ChronometerData.CHRONOMETER_STATE_STOPPED:
        chronometer.reset();
        break;
    }
    updateUI();
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);

    app = (ChronometerApplication) getApplication();
    chronometer = app.getChronometerData();

    counter = (TextView) findViewById(R.id.counter_view);
    lastLapAbsolute = (TextView) findViewById(R.id.last_lap);
    startPause = (Button) findViewById(R.id.start_pause);
    lapReset = (Button) findViewById(R.id.lap_reset);
    viewLaps = (Button) findViewById(R.id.view_laps);
    settings = (Button) findViewById(R.id.settings);

    startPause.setOnClickListener(this);
    lapReset.setOnClickListener(this);
    viewLaps.setOnClickListener(this);
    settings.setOnClickListener(this);

    myHandler = new Handler();
    runnable = new UpdateStopwatch();

    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);

    dialog = dialogBuilder
      .setTitle(R.string.pref_precision_title)
      .setView(R.layout.alert_preference)
      .setCancelable(true)
      .setNegativeButton(R.string.cancel, null)
      .create();

    // la activity registra un listener per
    // sapere quando le preferenze vengono cambiate
    SharedPreferences sharedPreferences =
      PreferenceManager.getDefaultSharedPreferences(this);
    sharedPreferences.registerOnSharedPreferenceChangeListener(this);
  }

  @Override
  public void onClick(View v) {
    switch (v.getId()) {
      case R.id.start_pause:
        onStartPauseClick();
        break;
      case R.id.lap_reset:
        onLapResetClick();
        break;
      case R.id.view_laps:
        Intent intent = new Intent(this, LapsActivity.class);
        startActivity(intent);
        break;
      case R.id.settings:
        openPreferences();
        break;
    }
  }

  // quando l'activity non è più visibile, la routine che
  // aggiorna l'interfaccia grafica non viene più invocata
  @Override
  public void onStop() {
    super.onStop();
    if (chronometer.getStatus() == ChronometerData.CHRONOMETER_STATE_RUNNING)
      myHandler.removeCallbacks(runnable);
  }

  // quando l'activity è almeno parzialmente visibile,
  // se il cronometro è in esecuzione, vengono
  // riattivate le invocazioni della routine
  @Override
  public void onStart() {
    super.onStart();
    if (chronometer.getStatus() == ChronometerData.CHRONOMETER_STATE_RUNNING)
      myHandler.postDelayed(runnable, UPDATE_INTERVAL);

    updateUI();
  }


  // ------------------------ Gestione delle preferenze ------------------------ //

  // classe per la gestione del click su uno dei radio-button
  private class RadioButtonChangeListener implements RadioGroup.OnCheckedChangeListener {

    @Override
    public void onCheckedChanged(RadioGroup radioGroup, int id) {
      int currentPrecision = 2;

      switch(id) {
        case R.id.pref_precision_1:
          currentPrecision = 1;
          break;
        case R.id.pref_precision_2:
          currentPrecision = 2;
          break;
        case R.id.pref_precision_3:
          currentPrecision = 3;
          break;
      }

      app.setChronometerPrecision(currentPrecision);

      // dopo aver cambiato valore, la finestra viene chiusa
      dialog.cancel();
    }
  }

  // mostra la finestra delle preferenze e registra
  // un nuovo listener per i radio-button
  void openPreferences() {
    dialog.show();

    RadioGroup radiogroup = (RadioGroup)
      dialog.findViewById(R.id.pref_radio_group);

    int currentPrecision = chronometer.getPrecision();
    RadioButton currentButton = (RadioButton)
      radiogroup.getChildAt(currentPrecision - 1);
    currentButton.setChecked(true);

    radiogroup.setOnCheckedChangeListener(new RadioButtonChangeListener());
  }

  // invocato quando viene modificato il valore della precisione corrente,
  // la activity aggiorna la propria interfaccia
  @Override
  public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String s) {
    updateUI();
  }
}